package com.cardiff.client_project.service.imp;

import com.cardiff.client_project.mapper.NursingHomeMapper;
import com.cardiff.client_project.pojo.dto.HospitalDTO;
import com.cardiff.client_project.service.NursingHomeService;
import com.cardiff.client_project.utils.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.dao.DataAccessException;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@Transactional
public class NursingHomeServiceImp implements NursingHomeService {

    private static final String STATUS_PENDING = "PENDING";
    private static final String STATUS_APPROVED = "APPROVED";
    private static final String STATUS_REJECTED = "REJECTED";

    private final NursingHomeMapper nursingHomeMapper;

    @Autowired
    public NursingHomeServiceImp(NursingHomeMapper nursingHomeMapper) {
        this.nursingHomeMapper = nursingHomeMapper;
    }

    @Override
    @Transactional(readOnly = true)
    public Result<List<HospitalDTO>> getAvailableBeds() {
        try {
            List<HospitalDTO> hospitals = nursingHomeMapper.findAllNursingHomes();
            hospitals = hospitals.stream()
                .filter(h -> STATUS_APPROVED.equals(h.getApprovalStatus()))
                .collect(Collectors.toList());
                
            if (hospitals.isEmpty()) {
                return Result.error("No approved hospitals available");
            }
            return Result.success(hospitals);
        } catch (DataAccessException e) {
            return Result.error("Database error: " + e.getMessage());
        } catch (Exception e) {
            log.error("Error fetching available beds: {}", e.getMessage(), e);
            return Result.error("Failed to fetch available beds");
        }
    }

    @Override
    @Transactional
    public Result<HospitalDTO> insertPatientInfo(HospitalDTO hospitalDTO) {
        try {
            if (!validateHospitalData(hospitalDTO)) {
                return Result.error("Invalid hospital data");
            }
            hospitalDTO.setApprovalStatus(STATUS_PENDING);
            int result = nursingHomeMapper.insertHospital(hospitalDTO);
            if (result > 0) {
                log.info("Successfully inserted hospital: {}", hospitalDTO.getName());
                return Result.success(hospitalDTO);
            }
            return Result.error("Failed to insert hospital");
        } catch (DataAccessException e) {
            log.error("Database error while inserting hospital: {}", e.getMessage(), e);
            return Result.error("Database error occurred");
        }
    }

    @Override
    public Result<String> updatePatient(HospitalDTO hospitalDTO) {
        try {
            if (!validateHospitalData(hospitalDTO)) {
                return Result.error("Invalid hospital data");
            }
            int result = nursingHomeMapper.updateHospital(hospitalDTO);
            return result > 0 ? 
                Result.success("Hospital updated successfully") :
                Result.error("Failed to update hospital");
        } catch (Exception e) {
            return Result.error("Error updating hospital: " + e.getMessage());
        }
    }

    @Override
    public Result<String> deletePatientById(List<Integer> ids) {
        try {
            int[] results = nursingHomeMapper.deleteHospitalByIds(ids);
            return results.length > 0 ? 
                Result.success("Hospital deleted successfully") :
                Result.error("No hospitals were deleted");
        } catch (Exception e) {
            return Result.error("Error deleting hospital: " + e.getMessage());
        }
    }

    @Override
    public Result<List<HospitalDTO>> getAllPatients() {
        return getAvailableBeds();
    }

    @Override
    public Result<String> updateBedCount(int hospitalId, int currentPatients) {
        try {
            if (currentPatients < 0) {
                return Result.error("Patient count cannot be negative");
            }
            int result = nursingHomeMapper.updateBedCount(hospitalId, currentPatients);
            return result > 0 ? 
                Result.success("Bed count updated successfully") :
                Result.error("Failed to update bed count");
        } catch (Exception e) {
            return Result.error("Error updating bed count: " + e.getMessage());
        }
    }

    @Override
    @Transactional(readOnly = true)
    public Result<List<HospitalDTO>> searchHospitals(String name) {
        try {
            if (name == null || name.trim().isEmpty()) {
                return Result.error("Search term cannot be empty");
            }
            List<HospitalDTO> hospitals = nursingHomeMapper.searchHospitals(name);
            return hospitals.isEmpty() ? 
                Result.error("No hospitals found") : 
                Result.success(hospitals);
        } catch (Exception e) {
            log.error("Search error: ", e);
            return Result.error("Search failed");
        }
    }

    @Override
    @Transactional
    public Result<String> updateApprovalStatus(Integer hospitalId, String status) {
        try {
            log.info("Updating approval status for hospital {}: {}", hospitalId, status);
            
            if (hospitalId == null || hospitalId <= 0) {
                log.warn("Invalid hospital ID: {}", hospitalId);
                return Result.error("Invalid hospital ID");
            }
            
            if (!isValidApprovalStatus(status)) {
                log.warn("Invalid approval status: {}", status);
                return Result.error("Invalid approval status");
            }
            
            int result = nursingHomeMapper.updateApprovalStatus(hospitalId, status.toUpperCase());
            if (result > 0) {
                log.info("Successfully updated approval status for hospital {}", hospitalId);
                return Result.success("Approval status updated successfully");
            }
            
            log.warn("Hospital not found with ID: {}", hospitalId);
            return Result.error("Hospital not found");
            
        } catch (DataAccessException e) {
            log.error("Database error while updating approval status: {}", e.getMessage(), e);
            return Result.error("Database error occurred");
        } catch (Exception e) {
            log.error("Unexpected error while updating approval status: {}", e.getMessage(), e);
            return Result.error("Failed to update approval status");
        }
    }

    @Override
    @Transactional(readOnly = true)
    public Result<List<HospitalDTO>> getPendingApprovals() {
        try {
            log.info("Fetching pending approval hospitals");
            List<HospitalDTO> pendingHospitals = nursingHomeMapper.getPendingApprovals();
            
            if (pendingHospitals.isEmpty()) {
                log.info("No pending approvals found");
                return Result.error("No pending approvals");
            }
            
            log.info("Found {} pending approvals", pendingHospitals.size());
            return Result.success(pendingHospitals);
        } catch (DataAccessException e) {
            log.error("Database error while fetching pending approvals: {}", e.getMessage(), e);
            return Result.error("Database error occurred");
        } catch (Exception e) {
            log.error("Unexpected error while fetching pending approvals: {}", e.getMessage(), e);
            return Result.error("Failed to fetch pending approvals");
        }
    }

    private boolean validateHospitalData(HospitalDTO hospitalDTO) {
        if (hospitalDTO == null) return false;
        return hospitalDTO.getName() != null &&
               !hospitalDTO.getName().trim().isEmpty() &&
               hospitalDTO.getLocation() != null &&
               !hospitalDTO.getLocation().trim().isEmpty() &&
               hospitalDTO.getPhone() != null &&
               !hospitalDTO.getPhone().trim().isEmpty() &&
               hospitalDTO.getTotalBeds() > 0 &&
               hospitalDTO.getAvailableBeds() >= 0 &&
               hospitalDTO.getAvailableBeds() <= hospitalDTO.getTotalBeds();
    }

    private boolean isValidStatus(String status) {
        return status != null && 
               (status.equals("APPROVED") || 
                status.equals("REJECTED") || 
                status.equals("PENDING"));
    }

    private boolean isValidApprovalStatus(String status) {
        if (status == null) return false;
        return List.of(STATUS_PENDING, STATUS_APPROVED, STATUS_REJECTED)
                   .contains(status.toUpperCase());
    }
}