package com.cardiff.client_project.controller.nursinghome;

import com.cardiff.client_project.pojo.dto.HospitalDTO;
import com.cardiff.client_project.service.NursingHomeService;
import com.cardiff.client_project.utils.Result;
import com.fasterxml.jackson.databind.ObjectMapper; // Needed for POST/PUT JSON conversion
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc; // Add this import
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.Collections;
import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import static org.hamcrest.Matchers.*; // For jsonPath checks
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print; // Add this import

// Use @WebMvcTest to test only the controller layer, mocking dependencies
@WebMvcTest(NursingHomeController.class)
@AutoConfigureMockMvc(addFilters = false) // <--- ADD THIS LINE TO DISABLE SECURITY FILTERS FOR THE TEST
public class NursingHomeControllerBurTest {

    @Autowired
    private MockMvc mockMvc; // Allows sending HTTP requests to the controller

    @Autowired
    private ObjectMapper objectMapper; // Utility to convert objects to JSON for POST/PUT

    @MockBean // Creates a Mockito mock for the service layer
    private NursingHomeService nursingHomeService;

    private HospitalDTO sampleHospitalDTO;

    @BeforeEach
    void setUp() {
        // Create a sample DTO for testing
        sampleHospitalDTO = new HospitalDTO();
        sampleHospitalDTO.setId(1);
        sampleHospitalDTO.setName("Test Hospital");
        sampleHospitalDTO.setLocation("Test Location");
        sampleHospitalDTO.setPhone("1234567890");
        sampleHospitalDTO.setTotalBeds(100);
        sampleHospitalDTO.setAvailableBeds(50);
        sampleHospitalDTO.setApprovalStatus("APPROVED"); // Assuming default is approved for some tests
    }

    @Test
    void testGetAvailableBeds_ReturnsOkAndData() throws Exception {
        // Arrange: Define what the mocked service should return
        List<HospitalDTO> hospitalList = Collections.singletonList(sampleHospitalDTO);
        Result<List<HospitalDTO>> successResult = Result.success(hospitalList);
        Mockito.when(nursingHomeService.getAvailableBeds()).thenReturn(successResult);

        // Act & Assert: Perform the GET request and check the response
        mockMvc.perform(get("/api/hospitals/available")
                .accept(MediaType.APPLICATION_JSON)) // Specify we accept JSON
               .andDo(print()) // <--- ADD THIS LINE TO PRINT THE RESPONSE
               .andExpect(status().isOk()) // Check for HTTP 200 OK
               .andExpect(content().contentType(MediaType.APPLICATION_JSON)) // Check content type
               .andExpect(jsonPath("$.code", is(1))) // Check Result success code
               .andExpect(jsonPath("$.msg").doesNotExist()) // Success shouldn't have a message
               .andExpect(jsonPath("$.data", hasSize(1))) // Check if data array has 1 item
               .andExpect(jsonPath("$.data[0].id", is(1)))
               .andExpect(jsonPath("$.data[0].name", is("Test Hospital"))); // Check the name
    }

    @Test
    void testGetAvailableBeds_HandlesServiceError() throws Exception {
        // Arrange: Define what the mocked service should return on error
        Result<List<HospitalDTO>> errorResult = Result.error("Database unavailable");
        Mockito.when(nursingHomeService.getAvailableBeds()).thenReturn(errorResult);

        // Act & Assert: Perform the GET request and check the error response
        mockMvc.perform(get("/api/hospitals/available")
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk()) // Controller still returns OK, but Result contains error
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(0))) // Check Result error code
               .andExpect(jsonPath("$.msg", is("Database unavailable"))) // Check error message
               .andExpect(jsonPath("$.data").doesNotExist()); // Error shouldn't have data
    }

    @Test
    void testSearchHospitals_WithValidName_ReturnsOkAndData() throws Exception {
        // Arrange
        List<HospitalDTO> hospitalList = Collections.singletonList(sampleHospitalDTO);
        Result<List<HospitalDTO>> successResult = Result.success(hospitalList);
        Mockito.when(nursingHomeService.searchHospitals(Mockito.eq("Test"))).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(get("/api/hospitals/search").param("name", "Test")
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.data", hasSize(1)))
               .andExpect(jsonPath("$.data[0].name", is("Test Hospital")));
    }

    @Test
    void testSearchHospitals_WithEmptyName_CallsGetAvailableBeds() throws Exception {
        // Arrange: Mock the getAvailableBeds call which should be triggered
        List<HospitalDTO> hospitalList = Collections.singletonList(sampleHospitalDTO);
        Result<List<HospitalDTO>> successResult = Result.success(hospitalList);
        Mockito.when(nursingHomeService.getAvailableBeds()).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(get("/api/hospitals/search").param("name", "") // Empty name
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.data", hasSize(1)))
               .andExpect(jsonPath("$.data[0].name", is("Test Hospital")));

        // Verify that searchHospitals was NOT called, but getAvailableBeds WAS called exactly once
        Mockito.verify(nursingHomeService, Mockito.never()).searchHospitals(Mockito.anyString());
        Mockito.verify(nursingHomeService, Mockito.times(1)).getAvailableBeds();
    }

     @Test
    void testDeleteHospital_ReturnsOk() throws Exception {
        // Arrange
        Result<String> successResult = Result.success("Hospital deleted successfully");
        // Ensure the mock matches the controller's call: deletePatientById(List.of(id))
        Mockito.when(nursingHomeService.deletePatientById(Mockito.eq(List.of(1)))).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(delete("/api/hospitals/{id}", 1) // Use path variable
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.msg", is("Hospital deleted successfully")))
               .andExpect(jsonPath("$.data").doesNotExist());
    }

    // --- Add tests for POST and PUT ---

    @Test
    void testAddHospital_ReturnsOkAndData() throws Exception {
        // Arrange
        // Prepare the DTO to be sent in the request body
        HospitalDTO requestDTO = new HospitalDTO();
        requestDTO.setName("New Hospital");
        requestDTO.setLocation("New Location");
        requestDTO.setPhone("9876543210");
        requestDTO.setTotalBeds(200);
        requestDTO.setAvailableBeds(150);
        // Note: The service sets the PENDING status, so we don't set it here

        // Mock the service response (it should return the DTO with PENDING status)
        HospitalDTO responseDTO = new HospitalDTO(); // Simulate what the service returns
        responseDTO.setId(2); // Assume DB assigns ID 2
        responseDTO.setName("New Hospital");
        responseDTO.setLocation("New Location");
        responseDTO.setPhone("9876543210");
        responseDTO.setTotalBeds(200);
        responseDTO.setAvailableBeds(150);
        responseDTO.setApprovalStatus("PENDING"); // Service sets this
        Result<HospitalDTO> successResult = Result.success(responseDTO);

        // Mock the service call - use Mockito.any(HospitalDTO.class) as the exact object might differ
        Mockito.when(nursingHomeService.insertPatientInfo(Mockito.any(HospitalDTO.class))).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(post("/api/hospitals")
                .contentType(MediaType.APPLICATION_JSON) // Set Content-Type for request body
                .content(objectMapper.writeValueAsString(requestDTO)) // Convert DTO to JSON string
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.data.id", is(2)))
               .andExpect(jsonPath("$.data.name", is("New Hospital")))
               .andExpect(jsonPath("$.data.approvalStatus", is("PENDING"))); // Verify status
    }

    @Test
    void testUpdateBedCount_ReturnsOk() throws Exception {
        // Arrange
        int hospitalId = 1;
        int currentPatients = 60;
        Result<String> successResult = Result.success("Bed count updated successfully");
        Mockito.when(nursingHomeService.updateBedCount(hospitalId, currentPatients)).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(put("/api/hospitals/{hospitalId}/beds", hospitalId)
                .param("currentPatients", String.valueOf(currentPatients)) // Send as request parameter
                .accept(MediaType.APPLICATION_JSON))
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.msg", is("Bed count updated successfully")));
    }

    // Add tests for /pending and /approval endpoints if needed

    @Test
    void testGetPendingApprovals_ReturnsOkAndData() throws Exception {
        // Arrange: Create a sample pending hospital DTO
        HospitalDTO pendingHospital = new HospitalDTO();
        pendingHospital.setId(2);
        pendingHospital.setName("Pending Hospital");
        pendingHospital.setLocation("Pending Location");
        pendingHospital.setPhone("5555555555");
        pendingHospital.setTotalBeds(50);
        pendingHospital.setAvailableBeds(50);
        pendingHospital.setApprovalStatus("PENDING");

        List<HospitalDTO> pendingList = Collections.singletonList(pendingHospital);
        Result<List<HospitalDTO>> successResult = Result.success(pendingList);
        Mockito.when(nursingHomeService.getPendingApprovals()).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(get("/api/hospitals/pending")
                .accept(MediaType.APPLICATION_JSON))
               .andDo(print()) // Print request/response for debugging
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.data", hasSize(1)))
               .andExpect(jsonPath("$.data[0].id", is(2)))
               .andExpect(jsonPath("$.data[0].name", is("Pending Hospital")))
               .andExpect(jsonPath("$.data[0].approvalStatus", is("PENDING")));
    }

    @Test
    void testGetPendingApprovals_HandlesServiceError() throws Exception {
        // Arrange: Mock service returning an error
        Result<List<HospitalDTO>> errorResult = Result.error("Failed to fetch pending approvals");
        Mockito.when(nursingHomeService.getPendingApprovals()).thenReturn(errorResult);

        // Act & Assert
        mockMvc.perform(get("/api/hospitals/pending")
                .accept(MediaType.APPLICATION_JSON))
               .andDo(print())
               .andExpect(status().isOk()) // Controller returns OK, Result contains error
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(0)))
               .andExpect(jsonPath("$.msg", is("Failed to fetch pending approvals")))
               .andExpect(jsonPath("$.data").doesNotExist());
    }

    @Test
    void testUpdateApprovalStatus_WithValidData_ReturnsOk() throws Exception {
        // Arrange
        int hospitalId = 2;
        String status = "APPROVED";
        Result<String> successResult = Result.success("Approval status updated successfully");
        Mockito.when(nursingHomeService.updateApprovalStatus(hospitalId, status)).thenReturn(successResult);

        // Act & Assert
        mockMvc.perform(put("/api/hospitals/{id}/approval", hospitalId)
                .param("status", status) // Send status as request parameter
                .accept(MediaType.APPLICATION_JSON))
               .andDo(print())
               .andExpect(status().isOk())
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(1)))
               .andExpect(jsonPath("$.msg", is("Approval status updated successfully")))
               .andExpect(jsonPath("$.data").doesNotExist());
    }

     @Test
    void testUpdateApprovalStatus_WithInvalidStatus_HandlesServiceError() throws Exception {
        // Arrange
        int hospitalId = 2;
        String invalidStatus = "MAYBE"; // Invalid status
        Result<String> errorResult = Result.error("Invalid approval status"); // Simulate service validation
        Mockito.when(nursingHomeService.updateApprovalStatus(hospitalId, invalidStatus)).thenReturn(errorResult);

        // Act & Assert
        mockMvc.perform(put("/api/hospitals/{id}/approval", hospitalId)
                .param("status", invalidStatus)
                .accept(MediaType.APPLICATION_JSON))
               .andDo(print())
               .andExpect(status().isOk()) // Controller returns OK, Result contains error
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(0)))
               .andExpect(jsonPath("$.msg", is("Invalid approval status")))
               .andExpect(jsonPath("$.data").doesNotExist());
    }

     @Test
    void testUpdateApprovalStatus_WhenHospitalNotFound_HandlesServiceError() throws Exception {
        // Arrange
        int nonExistentHospitalId = 999;
        String status = "APPROVED";
        Result<String> errorResult = Result.error("Hospital not found"); // Simulate service error
        Mockito.when(nursingHomeService.updateApprovalStatus(nonExistentHospitalId, status)).thenReturn(errorResult);

        // Act & Assert
        mockMvc.perform(put("/api/hospitals/{id}/approval", nonExistentHospitalId)
                .param("status", status)
                .accept(MediaType.APPLICATION_JSON))
               .andDo(print())
               .andExpect(status().isOk()) // Controller returns OK, Result contains error
               .andExpect(content().contentType(MediaType.APPLICATION_JSON))
               .andExpect(jsonPath("$.code", is(0)))
               .andExpect(jsonPath("$.msg", is("Hospital not found")))
               .andExpect(jsonPath("$.data").doesNotExist());
    }

} // End of class