package com.cardiff.client_project.service.imp;

import com.cardiff.client_project.mapper.NursingHomeMapper;
import com.cardiff.client_project.pojo.dto.HospitalDTO;
import com.cardiff.client_project.service.NursingHomeService;
import com.cardiff.client_project.utils.Result;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.dao.DataAccessResourceFailureException; // Example exception

import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*; // For assertions like assertEquals, assertTrue etc.

/**
 * Unit tests for the NursingHomeServiceImp class.
 * These tests focus ONLY on the service logic, mocking the mapper dependency.
 * No Spring context is loaded.
 */
@ExtendWith(MockitoExtension.class) // Initialize Mockito mocks
class NursingHomeServiceImpTest {

    @Mock // Create a mock instance of NursingHomeMapper
    private NursingHomeMapper nursingHomeMapper;

    @InjectMocks // Create an instance of NursingHomeServiceImp and inject the mocks (@Mock) into it
    private NursingHomeServiceImp nursingHomeService;

    private HospitalDTO approvedHospital;
    private HospitalDTO pendingHospital;

    @BeforeEach
    void setUp() {
        // Setup sample data before each test
        approvedHospital = new HospitalDTO();
        approvedHospital.setId(1);
        approvedHospital.setName("Approved Hospital");
        approvedHospital.setApprovalStatus("APPROVED");

        pendingHospital = new HospitalDTO();
        pendingHospital.setId(2);
        pendingHospital.setName("Pending Hospital");
        pendingHospital.setApprovalStatus("PENDING");
    }

    @Test
    void getAvailableBeds_WhenApprovedHospitalsExist_ReturnsSuccessWithData() {
        // Arrange: Define what the mock mapper should return when called
        List<HospitalDTO> allHospitals = List.of(approvedHospital, pendingHospital);
        Mockito.when(nursingHomeMapper.findAllNursingHomes()).thenReturn(allHospitals);

        // Act: Call the service method we are testing
        Result<List<HospitalDTO>> result = nursingHomeService.getAvailableBeds();

        // Assert: Check the outcome
        assertNotNull(result);
        assertEquals(1, result.getCode(), "Result code should be 1 for success");
        assertNotNull(result.getData());
        assertEquals(1, result.getData().size(), "Should only return the approved hospital");
        assertEquals("Approved Hospital", result.getData().get(0).getName());
        assertEquals("APPROVED", result.getData().get(0).getApprovalStatus());

        // Verify that the mapper method was called exactly once
        Mockito.verify(nursingHomeMapper, Mockito.times(1)).findAllNursingHomes();
    }

    @Test
    void getAvailableBeds_WhenNoApprovedHospitalsExist_ReturnsError() {
        // Arrange: Mapper returns only a pending hospital
        List<HospitalDTO> allHospitals = List.of(pendingHospital);
        Mockito.when(nursingHomeMapper.findAllNursingHomes()).thenReturn(allHospitals);

        // Act
        Result<List<HospitalDTO>> result = nursingHomeService.getAvailableBeds();

        // Assert
        assertNotNull(result);
        assertEquals(0, result.getCode(), "Result code should be 0 for error");
        assertEquals("No approved hospitals available", result.getMsg());
        assertNull(result.getData(), "Data should be null on error"); // Or check if it's an empty list depending on Result impl

        // Verify mapper was called
        Mockito.verify(nursingHomeMapper, Mockito.times(1)).findAllNursingHomes();
    }

    @Test
    void getAvailableBeds_WhenMapperReturnsEmptyList_ReturnsError() {
        // Arrange: Mapper returns an empty list
        Mockito.when(nursingHomeMapper.findAllNursingHomes()).thenReturn(Collections.emptyList());

        // Act
        Result<List<HospitalDTO>> result = nursingHomeService.getAvailableBeds();

        // Assert
        assertNotNull(result);
        assertEquals(0, result.getCode(), "Result code should be 0 for error");
        assertEquals("No approved hospitals available", result.getMsg());
        assertNull(result.getData());

        // Verify mapper was called
        Mockito.verify(nursingHomeMapper, Mockito.times(1)).findAllNursingHomes();
    }

    @Test
    void getAvailableBeds_WhenMapperThrowsException_ReturnsError() {
        // Arrange: Make the mock mapper throw an exception when called
        Mockito.when(nursingHomeMapper.findAllNursingHomes()).thenThrow(new DataAccessResourceFailureException("DB connection failed"));

        // Act
        Result<List<HospitalDTO>> result = nursingHomeService.getAvailableBeds();

        // Assert
        assertNotNull(result);
        assertEquals(0, result.getCode(), "Result code should be 0 for error");
        // Check for the specific error message from the DataAccessException catch block
        assertEquals("Database error: DB connection failed", result.getMsg()); // <-- FIX THIS LINE
        assertNull(result.getData());

        // Verify mapper was called
        Mockito.verify(nursingHomeMapper, Mockito.times(1)).findAllNursingHomes();
    }

    // --- Add more tests for other methods like insertPatientInfo, updateBedCount etc. ---
    // Example for insertPatientInfo (Success case)
    @Test
    void insertPatientInfo_WithValidData_ReturnsSuccess() {
        // Arrange
        HospitalDTO newHospital = new HospitalDTO();
        newHospital.setName("New Valid Hospital");
        newHospital.setLocation("Valid Location");
        newHospital.setPhone("111222333");
        newHospital.setTotalBeds(10);
        newHospital.setAvailableBeds(5);
        // We expect the service to set the status to PENDING

        // Mock the mapper's insert method to return 1 (indicating success)
        Mockito.when(nursingHomeMapper.insertHospital(Mockito.any(HospitalDTO.class))).thenReturn(1);

        // Act
        Result<HospitalDTO> result = nursingHomeService.insertPatientInfo(newHospital);

        // Assert
        assertNotNull(result);
        assertEquals(1, result.getCode());
        assertNotNull(result.getData());
        assertEquals("PENDING", result.getData().getApprovalStatus(), "Approval status should be set to PENDING");
        assertEquals("New Valid Hospital", result.getData().getName());

        // Verify the mapper was called once with the correct object (or any DTO in this case)
        Mockito.verify(nursingHomeMapper, Mockito.times(1)).insertHospital(Mockito.any(HospitalDTO.class));
    }

     // Example for insertPatientInfo (Validation failure case)
    @Test
    void insertPatientInfo_WithInvalidData_ReturnsError() {
        // Arrange
        HospitalDTO invalidHospital = new HospitalDTO(); // Missing required fields like name
        invalidHospital.setTotalBeds(-5); // Invalid bed count

        // Act
        Result<HospitalDTO> result = nursingHomeService.insertPatientInfo(invalidHospital);

        // Assert
        assertNotNull(result);
        assertEquals(0, result.getCode());
        assertEquals("Invalid hospital data", result.getMsg());
        assertNull(result.getData());

        // Verify the mapper was NEVER called because validation failed first
        Mockito.verify(nursingHomeMapper, Mockito.never()).insertHospital(Mockito.any(HospitalDTO.class));
    }

}